==> gubby-0.5.4.tar.gz: gubby-0.5.4/src/folders.cpp <==
/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: folders.cpp,v 1.8 2004/12/15 11:41:15 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "folders.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <iomanip>

Folders::Folders() : total(0) {
	addtrigger("From ");
	addtrigger(" Subject: ");
	addtrigger("  Folder: ");
}

std::string Folders::chop_(std::string line, const std::string& needle) const {
	line.erase(0, needle.size());

	int n(0);

	for ( const char* p=line.c_str();
		p[n]!=' ' && p[n]!='\t' && p[n]!='\0';
		++n);

	line.erase(n, line.size()-n);
	return line;
}

//! Called by grep for each matched line
void Folders::found(const std::string& line, const int triggerno) {

	std::string chopped( chop_( line, gettrigger(triggerno)));
	mail m;
	Folder cur;

	switch ( triggerno ) {

	case 0: // From
		from_ = chopped;
		break;

	case 1: // Subject
		subj_ = chopmin_(line, gettrigger(triggerno));
		break;

	case 2: // Folder
		m.subj = subj_;
		m.from = from_;

		subj_ = "";
		from_ = "";

		cur.setname(chopped);
		cur.push(m);

		// Insert folder or insert mail in folder
		for ( Folders::iterator p(begin()); p != end(); ++p )
			if ( (*p) == cur ) {
				(*p).push(m);
				newest_ = (*p).name();
				return;
			}

		newest_ = chopped;
		push_back(cur);

	default:
		break;
	}
}

unsigned long Folders::totalmail_() const {
	unsigned long ret(0);

	for ( Folders::const_iterator f = begin(); f != end(); ++f )
		ret += (*f).vmail.size();

	return ret;
}

void Folders::read(const std::string& file) {
	clear();
	
	std::ifstream f(file.c_str());
	find(f);

	sort(begin(), end());
	reverse(begin(), end());
	total = totalmail_();
}

#include <stdio.h>

//! this function is experimental, as it does no locking and does not
//! write a completely conformant procmail logfile (the numbers and dates
//! are missing)
void Folders::write(const std::string& procmail_logfile) {
	// TODO: make sure no changes has been made to logfile
	// from THIS position to right before the rename() call,
	// if so, then new mail has arrived since last time...

	std::string tmp = procmail_logfile + ".tmp";

	FILE* f = fopen(tmp.c_str(), "wb");
	if ( f ) {
		for ( Folders::const_iterator p = begin(); p != end(); ++p ) {
			for ( std::vector<mail>::const_iterator m =
				p->vmail.begin(); m != p->vmail.end(); ++m )
			{
				fprintf(f, "From %s\n", m->from.c_str());
				fprintf(f, " Subject: %s\n", m->subj.c_str());
				fprintf(f, "  Folder: %s\n", p->name().c_str());
			}
		}
		fclose(f);
		rename(tmp.c_str(), procmail_logfile.c_str());
	}
}

void Folders::remove(const std::string& foldername) {
	for ( Folders::iterator p = begin(); p < end(); ++p )
		if ( foldername == p->name() )
			erase(p);

	sort(begin(), end());
	reverse(begin(), end());
	total = totalmail_();
}

==> gubby-0.5.5.tar.gz: gubby-0.5.5/src/folders.cpp <==
/*!
**    Notice:  Gubby Copyright 2003 Christian Stigen Larsen
**    Author:  <csl@sublevel3.org>
**
**    $Id: folders.cpp,v 1.10 2004/12/17 14:29:48 cslsublevel3org Exp $
**
**    This file is part of Gubby.
**
**    Gubby is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    Gubby is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with Gubby; if not, write to the Free Software
**    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

#include "folders.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <iomanip>

Folders::Folders() : total(0) {
	addtrigger("From ");
	addtrigger(" Subject: ");
	addtrigger("  Folder: ");
}

std::string Folders::chop_(std::string line, const std::string& needle) const {
	line.erase(0, needle.size());

	int n(0);

	for ( const char* p=line.c_str();
		p[n]!=' ' && p[n]!='\t' && p[n]!='\0';
		++n);

	line.erase(n, line.size()-n);
	return line;
}

//! Called by grep for each matched line
void Folders::found(const std::string& line, const int triggerno) {
	using namespace std;

	string chopped( chop_( line, gettrigger(triggerno)));
	mail m;
	Folder cur;
	string::size_type trim;

	switch ( triggerno ) {

	case 0: // From
		from_ = chopped;
		break;

	case 1: // Subject
		subj_ = chopmin_(line, gettrigger(triggerno));
		break;

	case 2: // Folder
		m.subj = subj_;
		m.from = from_;

		subj_ = "";
		from_ = "";

		if ( (trim = chopped.find("/new/", 0)) != string::npos )
			chopped.erase(trim, chopped.length()-trim);

		cur.setname(chopped);
		cur.push(m);

		// Insert folder or insert mail in folder
		for ( Folders::iterator p(begin()); p != end(); ++p )
			if ( (*p) == cur ) {
				(*p).push(m);
				newest_ = (*p).name();
				return;
			}

		newest_ = chopped;
		push_back(cur);

	default:
		break;
	}
}

unsigned long Folders::totalmail_() const {
	unsigned long ret(0);

	for ( Folders::const_iterator f = begin(); f != end(); ++f )
		ret += (*f).vmail.size();

	return ret;
}

void Folders::read(const std::string& file) {
	clear();
	
	std::ifstream f(file.c_str());
	find(f);

	sort(begin(), end());
	reverse(begin(), end());
	total = totalmail_();
}

#include <stdio.h>

//! this function is experimental, as it does no locking and does not
//! write a completely conformant procmail logfile (the numbers and dates
//! are missing)
void Folders::write(const std::string& procmail_logfile) {
	// TODO: make sure no changes has been made to logfile
	// from THIS position to right before the rename() call,
	// if so, then new mail has arrived since last time...

	std::string tmp = procmail_logfile + ".tmp";

	FILE* f = fopen(tmp.c_str(), "wb");
	if ( f ) {
		for ( Folders::const_iterator p = begin(); p != end(); ++p ) {
			for ( std::vector<mail>::const_iterator m =
				p->vmail.begin(); m != p->vmail.end(); ++m )
			{
				fprintf(f, "From %s\n", m->from.c_str());
				fprintf(f, " Subject: %s\n", m->subj.c_str());
				fprintf(f, "  Folder: %s\n", p->name().c_str());
			}
		}
		fclose(f);
		rename(tmp.c_str(), procmail_logfile.c_str());
	}
}

void Folders::remove(const std::string& foldername) {
	for ( Folders::iterator p = begin(); p < end(); ++p )
		if ( foldername == p->name() )
			erase(p);

	sort(begin(), end());
	reverse(begin(), end());
	total = totalmail_();
}
